# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-24 04:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('river', '0005_auto_20160428_1818'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proceedingmeta',
            name='groups',
            field=models.ManyToManyField(blank=True, null=True, to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='proceedingmeta',
            name='order',
            field=models.IntegerField(default=0, null=True, verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='proceedingmeta',
            name='parents',
            field=models.ManyToManyField(blank=True, db_index=True, null=True, related_name='children', to='river.ProceedingMeta', verbose_name=b'parents'),
        ),
        migrations.AlterField(
            model_name='proceedingmeta',
            name='permissions',
            field=models.ManyToManyField(blank=True, to='auth.Permission', verbose_name='Permissions'),
        ),
        migrations.AlterField(
            model_name='transition',
            name='direction',
            field=models.SmallIntegerField(choices=[(0, 'Backward'), (1, 'Forward')], default=1, verbose_name='Transition Direction'),
        ),
    ]
