# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-24 11:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('river', '0008_auto_20161017_0916'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proceedingmeta',
            name='groups',
            field=models.ManyToManyField(blank=True, to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='proceedingmeta',
            name='parents',
            field=models.ManyToManyField(blank=True, db_index=True, related_name='children', to='river.ProceedingMeta', verbose_name='parents'),
        ),
    ]
