# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-28 18:18
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('river', '0004_data_fix'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='proceedingtrack',
            name='previous_track',
        ),
        migrations.RemoveField(
            model_name='proceedingtrack',
            name='proceeding',
        ),
        migrations.AddField(
            model_name='proceeding',
            name='cloned',
            field=models.BooleanField(default=False, verbose_name='Cloned?'),
        ),
        migrations.AddField(
            model_name='proceeding',
            name='previous',
            field=mptt.fields.TreeOneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='next_proceeding', to='river.Proceeding', verbose_name='Previous Proceeding'),
        ),
        migrations.DeleteModel(
            name='ProceedingTrack',
        ),
    ]
