from django.db import models

from river.models.fields.state import StateField

__author__ = 'ahmetdal'


class TestModel(models.Model):
    test_field = models.CharField(max_length=50, null=True, blank=True)
    my_field = StateField()


class TestModelWithoutStateField(models.Model):
    test_field = models.CharField(max_length=50, null=True, blank=True)


class TestModelSlowCase1(models.Model):
    status = StateField()


class TestModelSlowCase2(models.Model):
    status = StateField()
