# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-17 09:16
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('river', '0007_auto_20161015_0959'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='proceedingmeta',
            unique_together=set([('content_type', 'transition', 'order')]),
        ),
        migrations.RemoveField(
            model_name='proceeding',
            name='field',
        ),
        migrations.RemoveField(
            model_name='proceedingmeta',
            name='field',
        ),
    ]
